@@ -16,7 +16,7 @@
  * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <xfs/libxfs.h>
+#include "libxfs.h"
 #include "command.h"
 #include "freesp.h"
 #include "io.h"
@@ -54,6 +54,7 @@ static int	usage(void);
 
 static int		agcount;
 static xfs_agnumber_t	*aglist;
+static int		alignment;
 static int		countflag;
 static int		dumpflag;
 static int		equalsize;
@@ -67,7 +68,7 @@ static long long	totexts;
 
 static const cmdinfo_t	freesp_cmd =
 	{ "freesp", NULL, freesp_f, 0, -1, 0,
-	  "[-bcdfs] [-a agno]... [-e binsize] [-h h1]... [-m binmult]",
+	  "[-bcdfs] [-A alignment] [-a agno]... [-e binsize] [-h h1]... [-m binmult]",
 	  "summarize free space for filesystem", NULL };
 
 static int
@@ -96,6 +97,10 @@ freesp_f(
 
 	if (!init(argc, argv))
 		return 0;
+
+	if (dumpflag)
+		dbprintf("%8s %8s %8s\n", "agno", "agbno", "len");
+
 	for (agno = 0; agno < mp->m_sb.sb_agcount; agno++)  {
 		if (inaglist(agno))
 			scan_ag(agno);
@@ -143,8 +148,11 @@ init(
 	totblocks = totexts = 0;
 	aglist = NULL;
 	hist = NULL;
-	while ((c = getopt(argc, argv, "a:bcde:h:m:s")) != EOF) {
+	while ((c = getopt(argc, argv, "A:a:bcde:h:m:s")) != EOF) {
 		switch (c) {
+		case 'A':
+			alignment = atoi(optarg);
+			break;
 		case 'a':
 			aglistadd(optarg);
 			break;
@@ -231,6 +239,7 @@ scan_freelist(
 	xfs_agfl_t	*agfl;
 	xfs_agblock_t	bno;
 	int		i;
+	__be32		*agfl_bno;
 
 	if (be32_to_cpu(agf->agf_flcount) == 0)
 		return;
@@ -240,9 +249,13 @@ scan_freelist(
 	agfl = iocur_top->data;
 	i = be32_to_cpu(agf->agf_flfirst);
 
+	/* open coded XFS_BUF_TO_AGFL_BNO */
+	agfl_bno = xfs_sb_version_hascrc(&mp->m_sb) ? &agfl->agfl_bno[0]
+						   : (__be32 *)agfl;
+
 	/* verify agf values before proceeding */
-	if (be32_to_cpu(agf->agf_flfirst) >= XFS_AGFL_SIZE(mp) ||
-	    be32_to_cpu(agf->agf_fllast) >= XFS_AGFL_SIZE(mp)) {
+	if (be32_to_cpu(agf->agf_flfirst) >= libxfs_agfl_size(mp) ||
+	    be32_to_cpu(agf->agf_fllast) >= libxfs_agfl_size(mp)) {
 		dbprintf(_("agf %d freelist blocks bad, skipping "
 			  "freelist scan\n"), i);
 		pop_cur();
@@ -250,11 +263,11 @@ scan_freelist(
 	}
 
 	for (;;) {
-		bno = be32_to_cpu(agfl->agfl_bno[i]);
+		bno = be32_to_cpu(agfl_bno[i]);
 		addtohist(seqno, bno, 1);
 		if (i == be32_to_cpu(agf->agf_fllast))
 			break;
-		if (++i == XFS_AGFL_SIZE(mp))
+		if (++i == libxfs_agfl_size(mp))
 			i = 0;
 	}
 	pop_cur();
@@ -296,7 +309,8 @@ scanfunc_bno(
 	xfs_alloc_ptr_t		*pp;
 	xfs_alloc_rec_t		*rp;
 
-	if (be32_to_cpu(block->bb_magic) != XFS_ABTB_MAGIC)
+	if (!(be32_to_cpu(block->bb_magic) == XFS_ABTB_MAGIC ||
+	      be32_to_cpu(block->bb_magic) == XFS_ABTB_CRC_MAGIC))
 		return;
 
 	if (level == 0) {
@@ -323,7 +337,8 @@ scanfunc_cnt(
 	xfs_alloc_ptr_t		*pp;
 	xfs_alloc_rec_t		*rp;
 
-	if (be32_to_cpu(block->bb_magic) != XFS_ABTC_MAGIC)
+	if (!(be32_to_cpu(block->bb_magic) == XFS_ABTC_MAGIC ||
+	      be32_to_cpu(block->bb_magic) == XFS_ABTC_CRC_MAGIC))
 		return;
 
 	if (level == 0) {
@@ -361,6 +376,9 @@ addtohist(
 {
 	int		i;
 
+	if (alignment && (XFS_AGB_TO_FSB(mp,agno,agbno) % alignment))
+		return;
+
 	if (dumpflag)
 		dbprintf("%8d %8d %8d\n", agno, agbno, len);
 	totexts++;
