@@ -705,6 +705,7 @@ _("%s freespace btree block claimed (state %d), agno %d, bno %d, suspect %d\n"),
 							     XR_E_FREE);
 						break;
 					}
+					/* fall through */
 				default:
 					do_warn(
 	_("block (%d,%d-%d) multiply claimed by %s space tree, state - %d\n"),
@@ -2143,8 +2144,8 @@ scan_freelist(
 
 	if (no_modify) {
 		/* agf values not fixed in verify_set_agf, so recheck */
-		if (be32_to_cpu(agf->agf_flfirst) >= XFS_AGFL_SIZE(mp) ||
-		    be32_to_cpu(agf->agf_fllast) >= XFS_AGFL_SIZE(mp)) {
+		if (be32_to_cpu(agf->agf_flfirst) >= libxfs_agfl_size(mp) ||
+		    be32_to_cpu(agf->agf_fllast) >= libxfs_agfl_size(mp)) {
 			do_warn(_("agf %d freelist blocks bad, skipping "
 				  "freelist scan\n"), i);
 			return;
@@ -2162,7 +2163,7 @@ scan_freelist(
 		count++;
 		if (i == be32_to_cpu(agf->agf_fllast))
 			break;
-		if (++i == XFS_AGFL_SIZE(mp))
+		if (++i == libxfs_agfl_size(mp))
 			i = 0;
 	}
 	if (count != be32_to_cpu(agf->agf_flcount)) {
@@ -2342,7 +2343,7 @@ validate_agi(
  */
 static void
 scan_ag(
-	work_queue_t	*wq,
+	struct workqueue*wq,
 	xfs_agnumber_t	agno,
 	void		*arg)
 {
@@ -2504,13 +2505,13 @@ scan_ags(
 	struct xfs_mount	*mp,
 	int			scan_threads)
 {
-	struct aghdr_cnts *agcnts;
-	uint64_t	fdblocks = 0;
-	uint64_t	icount = 0;
-	uint64_t	ifreecount = 0;
-	uint64_t	usedblocks = 0;
-	xfs_agnumber_t	i;
-	work_queue_t	wq;
+	struct aghdr_cnts	*agcnts;
+	uint64_t		fdblocks = 0;
+	uint64_t		icount = 0;
+	uint64_t		ifreecount = 0;
+	uint64_t		usedblocks = 0;
+	xfs_agnumber_t		i;
+	struct workqueue	wq;
 
 	agcnts = malloc(mp->m_sb.sb_agcount * sizeof(*agcnts));
 	if (!agcnts) {
